@@ -61,11 +61,16 @@ local counters = allocate {
         0x0074, 0x0075, 0x0076, 0x007A, 0x017E
     },
     ['greek'] = { -- this should be the lowercase table
-        0x0391, 0x0392, 0x0393, 0x0394, 0x0395,
-        0x0396, 0x0397, 0x0398, 0x0399, 0x039A,
-        0x039B, 0x039C, 0x039D, 0x039E, 0x039F,
-        0x03A0, 0x03A1, 0x03A3, 0x03A4, 0x03A5,
-        0x03A6, 0x03A7, 0x03A8, 0x03A9
+     -- 0x0391, 0x0392, 0x0393, 0x0394, 0x0395,
+     -- 0x0396, 0x0397, 0x0398, 0x0399, 0x039A,
+     -- 0x039B, 0x039C, 0x039D, 0x039E, 0x039F,
+     -- 0x03A0, 0x03A1, 0x03A3, 0x03A4, 0x03A5,
+     -- 0x03A6, 0x03A7, 0x03A8, 0x03A9
+        0x03B1, 0x03B2, 0x03B3, 0x03B4, 0x03B5,
+        0x03B6, 0x03B7, 0x03B8, 0x03B9, 0x03BA,
+        0x03BB, 0x03BC, 0x03BD, 0x03BE, 0x03BF,
+        0x03C0, 0x03C1, 0x03C3, 0x03C4, 0x03C5,
+        0x03C6, 0x03C7, 0x03C8, 0x03C9,
     },
     ['arabic'] = {
         0x0627, 0x0628, 0x062C, 0x062F, 0x0647,
@@ -298,7 +303,7 @@ local function toroman(n)
     if n >= 4000 then
         return toroman(floor(n/1000)) .. " " .. toroman(n%1000)
     else
-        return rep("M",floor(n/1000)) .. roman[3][floor((n%1000)/100)] .. roman[2][floor((n%100)/10)] .. roman[1][floor((n% 10)/1)]
+        return rep("M",floor(n/1000)) .. roman[3][floor((n%1000)/100)] .. roman[2][floor((n%100)/10)] .. roman[1][floor((n%10)/1)]
     end
 end
 
@@ -705,7 +710,7 @@ end
 local ordinals = {
     english = function(n)
         local two = n % 100
-        if two == 11 or two == 12 or to == 13 then
+        if two == 11 or two == 12 or two == 13 then
             return "th"
         else
             local one = n % 10
@@ -742,6 +747,7 @@ end
 function commands.ordinal(n,language)
     local t = language and ordinals[language]
     local o = t and t(n)
+    context(n)
     if o then
         context.highordinalstr(o)
     end
@@ -758,29 +764,29 @@ local v_space    = variables.space
 
 local convert = converters.convert
 
-local days = {
-    variables.sunday,
-    variables.monday,
-    variables.tuesday,
-    variables.wednesday,
-    variables.thursday,
-    variables.friday,
-    variables.saturday,
+local days = { -- not variables.sunday
+    "sunday",
+    "monday",
+    "tuesday",
+    "wednesday",
+    "thursday",
+    "friday",
+    "saturday",
 }
 
-local months = {
-     variables.january,
-     variables.february,
-     variables.march,
-     variables.april,
-     variables.may,
-     variables.june,
-     variables.july,
-     variables.august,
-     variables.september,
-     variables.october,
-     variables.november,
-     variables.december,
+local months = { -- not variables.januari
+     "january",
+     "february",
+     "march",
+     "april",
+     "may",
+     "june",
+     "july",
+     "august",
+     "september",
+     "october",
+     "november",
+     "december",
 }
 
 function commands.dayname(n)
@@ -802,16 +808,16 @@ end
 
 -- a prelude to a function that we can use at the lua end
 
--- historic    : day+    month:mnem
--- maybe better: day:ord month:mmem
+-- day:ord month:mmem
+-- j and jj obsolete
 
-function commands.currentdate(str,currentlanguage) -- j and jj obsolete
+function commands.currentdate(str,currentlanguage) -- second argument false : no label
     local list = utilities.parsers.settings_to_array(str)
     local year, month, day = tex.year, tex.month, tex.day
     local auto = true
     for i=1,#list do
         local entry = list[i]
-        local tag, plus = match(entry,"^([^%+:]+)(.*)$")
+        local tag, plus = languages.labels.split(entry)
         local ordinal, mnemonic, whatordinal = false, false, nil
         if not tag then
             tag = entry
@@ -831,7 +837,9 @@ function commands.currentdate(str,currentlanguage) -- j and jj obsolete
         elseif tag == "Y" then
             context(year)
         elseif tag == v_month or tag == "m" then
-            if mnemonic then
+            if currentlanguage == false then
+                context(months[month] or "unknown")
+            elseif mnemonic then
                 commands.monthmnem(month)
             else
                 commands.monthname(month)
@@ -841,7 +849,11 @@ function commands.currentdate(str,currentlanguage) -- j and jj obsolete
         elseif tag == "M" then
             context(month)
         elseif tag == v_day or tag == "d" then
-            context.convertnumber(v_day,day)
+            if currentlanguage == false then
+                context(days[day] or "unknown")
+            else
+                context.convertnumber(v_day,day)
+            end
             whatordinal = day
         elseif tag == "dd" then
             context("%02i",day)
@@ -850,7 +862,12 @@ function commands.currentdate(str,currentlanguage) -- j and jj obsolete
             context(day)
             whatordinal = day
         elseif tag == v_weekday or tag == "w" then
-            commands.dayname(weekday(day,month,year))
+            local wd = weekday(day,month,year)
+            if currentlanguage == false then
+                context(days[wd] or "unknown")
+            else
+                commands.dayname(wd)
+            end
         elseif tag == "W" then
             context(weekday(day,month,year))
         elseif tag == v_referral then
@@ -863,7 +880,15 @@ function commands.currentdate(str,currentlanguage) -- j and jj obsolete
             auto = true
         end
         if ordinal and whatordinal then
-            commands.ordinal(whatordinal,currentlanguage)
+            if currentlanguage == false then
+                -- ignore
+            else
+                context("%s",converters.ordinal(whatordinal,currentlanguage))
+            end
         end
     end
 end
+
+function commands.rawdate(str)
+    commands.currentdate(str,false)
+end
