@@ -96,10 +96,11 @@ local validmethods      = table.tohash {
 }
 
 local predefinedmethods = {
-    [v_before] = "mm,mc,uc",
-    [v_after]  = "pm,mc,uc",
-    [v_first]  = "pc,mm,uc",
-    [v_last]   = "mc,mm,uc",
+    [v_default] = "zc,pc,zm,pm,uc",
+    [v_before]  = "mm,mc,uc",
+    [v_after]   = "pm,mc,uc",
+    [v_first]   = "pc,mm,uc",
+    [v_last]    = "mc,mm,uc",
 }
 
 sorters = {
@@ -113,7 +114,7 @@ sorters = {
         digitsoffset      = digitsoffset,
         digitsmaximum     = digitsmaximum,
         defaultlanguage   = v_default,
-        defaultmethod     = v_before,
+        defaultmethod     = v_default,
         defaultdigits     = v_numbers,
     }
 }
@@ -317,7 +318,7 @@ local function setlanguage(l,m,d,u)
     method = predefinedmethods[variables[method]] or method
     data.method  = method
     --
-    data.digits  = digite
+    data.digits  = digits
     --
     local seq = utilities.parsers.settings_to_array(method or "") -- check the list
     sequence = { }
