@@ -315,7 +315,8 @@ return {
         {
             filename = "typo-ini",
             marktype = "mkiv",
-            status   = "unknown",
+            status   = "okay",
+            comment  = "will grow",
         },
         {
             filename = "page-ins",
@@ -1106,7 +1107,8 @@ return {
         {
             filename = "typo-dir",
             marktype = "mkiv",
-            status   = "unknown",
+            status   = "okay",
+            comment  = "maybe singular setup"
         },
         {
             filename = "typo-brk",
@@ -1323,7 +1325,7 @@ return {
         {
             filename = "chem-str",
             marktype = "mkiv",
-            status   = "unknown",
+            status   = "okay",
         },
         {
             filename = "typo-scr",
