@@ -10,14 +10,29 @@ local structures = structures
 local itemgroups = structures.itemgroups
 local jobpasses  = job.passes
 
+local setfield   = jobpasses.save
+local getfield   = jobpasses.getfield
+
 function itemgroups.register(name,nofitems,maxwidth)
-    jobpasses.savedata("itemgroup", { nofitems, maxwidth })
+    setfield("itemgroup", { nofitems, maxwidth })
 end
 
 function itemgroups.nofitems(name,index)
-    jobpasses.getfield("itemgroup", index, 1, 0)
+    return getfield("itemgroup", index, 1, 0)
 end
 
 function itemgroups.maxwidth(name,index)
-    jobpasses.getfield("itemgroup", index, 2, 0)
+    return getfield("itemgroup", index, 2, 0)
+end
+
+-- interface (might become counter/dimension)
+
+commands.registeritemgroup = itemgroups.register
+
+function commands.nofitems(name,index)
+    context(getfield("itemgroup", index, 1, 0))
+end
+
+function commands.maxitemwidth(name,index)
+    context(getfield("itemgroup", index, 2, 0))
 end
